@@ -50,7 +50,7 @@
 #' along the sequence for which to compute submodels.  The default is to start 
 #' with a model containing only an intercept (step 0) and iteratively add all 
 #' variables along the sequence (step \code{sMax}).  If the second element is 
-#' \code{NA}, predictors may be added to the model as long as there are twice 
+#' \code{NA}, predictors are added to the model as long as there are twice 
 #' as many observations as predictors.  If only one value is supplied, it is 
 #' recycled.
 #' @param regFun  a function to compute robust linear regressions along the 
@@ -104,9 +104,9 @@
 #' If \code{fit} is \code{FALSE}, an integer vector containing the indices of 
 #' the sequenced predictors.
 #'  
-#' Else if \code{crit} is \code{"PE"}, an object of class \code{"perryRlars"} 
-#' (inheriting from classes \code{"perrySeqModel"} and \code{"perryTuning"}, 
-#' see \code{\link[perry]{perryTuning}}).  It contains information on the 
+#' Else if \code{crit} is \code{"PE"}, an object of class 
+#' \code{"perrySeqModel"} (inheriting from class \code{"perrySelect"}, 
+#' see \code{\link[perry]{perrySelect}}).  It contains information on the 
 #' prediction error criterion, and includes the final model as component 
 #' \code{finalModel}.
 #' 
@@ -169,6 +169,7 @@
 #' @import RcppArmadillo
 #' @import parallel
 #' @import pcaPP
+#' @import perry
 
 rlars <- function(x, ...) UseMethod("rlars")
 
@@ -247,33 +248,39 @@ rlars.default <- function(x, y, sMax = NA, centerFun = median,
   ## prediction error estimation
   fit <- isTRUE(fit)
   if(fit) {
-    s <- checkSRange(s, sMax=sMax)
-    if(s[1] > s[2]) s[1] <- s[2]
-    crit <- if(s[1] == s[2]) "none" else match.arg(crit)
+    s <- checkSRange(s, sMax=sMax)  # check range of steps along the sequence
+    crit <- if(!is.na(s[2]) && s[1] == s[2]) "none" else match.arg(crit)
     if(crit == "PE") {
-      # set up function call to be passed to perryTuning()
-      remove <- c("x", "y", "s", "crit", "splits", "cost", "costArgs", 
+      # further checks for steps along the sequence
+      if(is.na(s[2])) {
+        s[2] <- min(sMax, floor(n/2))
+        if(s[1] > sMax) s[1] <- sMax
+      }
+      # set up function call to be passed to perryFit()
+      remove <- c("x", "y", "crit", "splits", "cost", "costArgs", 
                   "selectBest", "seFactor", "ncores", "cl", "seed")
       remove <- match(remove, names(matchedCall), nomatch=0)
       call <- matchedCall[-remove]
-      # call function perryTuning() to perform prediction error estimation
+      call$sMax <- sMax
+      call$s <- s
+      # call function perryFit() to perform prediction error estimation
       s <- seq(from=s[1], to=s[2])
-      tuning <- list(s=s)
       selectBest <- match.arg(selectBest)
-      out <- perryTuning(call, x=x, y=y, tuning=tuning, splits=splits, 
-                         predictArgs=list(fit="both"), cost=cost, 
-                         costArgs=costArgs, selectBest=selectBest, 
-                         seFactor=seFactor, ncores=ncores, cl=cl, 
-                         seed=seed)
+      out <- perryFit(call, x=x, y=y, splits=splits, 
+                      predictArgs=list(s=s, recycle=TRUE), cost=cost, 
+                      costArgs=costArgs, envir=parent.frame(), 
+                      ncores=ncores, cl=cl, seed=seed)
+      out <- perryReshape(out, selectBest=selectBest, seFactor=seFactor)
+      fits(out) <- s
       # fit final model
       call$x <- matchedCall$x
       call$y <- matchedCall$y
       call$s <- s[out$best]
       call$ncores <- matchedCall$ncores
-      out$finalModel <- eval(call)
+      out$finalModel <- eval(call, envir=parent.frame())
       out$call <- matchedCall
       # assign class and return object
-      class(out) <- c("perryRlars", "perrySeqModel", class(out))
+      class(out) <- c("perrySeqModel", class(out))
       return(out)
     }
   }
@@ -354,8 +361,8 @@ rlars.default <- function(x, y, sMax = NA, centerFun = median,
     out[c("muX", "sigmaX", "muY", "sigmaY")] <- list(muX, sigmaX, muY, sigmaY)
     # add model data to result if requested
     if(isTRUE(model)) out[c("x", "y")] <- list(x=x, y=y)
-    if(winsorize) out$w <- w
-    out$call <- matchedCall  # add call to return object
+    if(winsorize) out$w <- w  # add data cleaning weights
+    out$call <- matchedCall   # add call to return object
     class(out) <- c("rlars", class(out))
     out
   } else active
