@@ -30,6 +30,10 @@
 #' @param alpha  a numeric value giving the percentage of the residuals for 
 #' which the \eqn{L_{1}}{L1} penalized sum of squares should be minimized (the 
 #' default is 0.75).
+#' @param normalize  a logical indicating whether the predictor variables 
+#' should be normalized to have unit \eqn{L_{2}}{L2} norm (the default is 
+#' \code{TRUE}).  Note that normalization is performed on the subsamples 
+#' rather than the full data set.
 #' @param intercept  a logical indicating whether a constant term should be 
 #' included in the model (the default is \code{TRUE}).
 #' @param nsamp  a numeric vector giving the number of subsamples to be used in 
@@ -170,16 +174,22 @@
 #' @returnItem y  the response variable (if \code{model} is \code{TRUE}).
 #' @returnItem call  the matched function call.
 #' 
-#' Package \pkg{robustHD} has a built-in back end for sparse least trimmed 
-#' squares using the C++ library Armadillo.  Another back end is available 
-#' through package \pkg{sparseLTSEigen}, which uses the C++ library Eigen.  The 
-#' latter is faster, currently does not work on 32-bit \R for Windows.
+#' @note Package \pkg{robustHD} has a built-in back end for sparse least 
+#' trimmed squares using the C++ library Armadillo.  Another back end is 
+#' available through package \pkg{sparseLTSEigen}, which uses the C++ library 
+#' Eigen.  The latter is faster, currently does not work on 32-bit \R for 
+#' Windows.
 #' 
 #' For both C++ back ends, parallel computing is implemented via OpenMP 
 #' (\url{http://openmp.org/}).
 #' 
 #' @author Andreas Alfons
 #' 
+#' @references
+#' Alfons, A., Croux, C. and Gelper, S. (2013) Sparse least trimmed squares 
+#' regression for analyzing high-dimensional large data sets. \emph{The Annals 
+#' of Applied Statistics}, \bold{7}(1), 226--248.
+#' 
 #' @seealso \code{\link[=coef.sparseLTS]{coef}}, 
 #' \code{\link[=fitted.sparseLTS]{fitted}}, 
 #' \code{\link[=plot.sparseLTS]{plot}}, 
@@ -240,7 +250,7 @@ sparseLTS.formula <- function(formula, data, ...) {
 #' @export
 
 sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"), 
-                              alpha = 0.75, intercept = TRUE, 
+                              alpha = 0.75, normalize = TRUE, intercept = TRUE, 
                               nsamp = c(500, 10), 
                               initial = c("sparse", "hyperplane", "random"), 
                               ncstep = 2, use.correction = TRUE, 
@@ -278,13 +288,15 @@ sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"),
   }
   if(length(lambda) == 1) crit <- "none" 
   else crit <- match.arg(crit)
+  normalize <- isTRUE(normalize)
   intercept <- isTRUE(intercept)
   if(mode == "fraction" && any(lambda > 0)) { 
     # fraction of a robust estimate of the smallest value for the penalty 
     # parameter that sets all coefficients to zero (based on bivariate 
     # winsorization)
     if(crit == "PE") frac <- lambda
-    lambda <- lambda * lambda0(x, y, intercept=intercept, tol=tol, eps=eps, ...)
+    lambda <- lambda * lambda0(x, y, normalize=normalize, intercept=intercept, 
+                               tol=tol, eps=eps, ...)
   }
   alpha <- rep(alpha, length.out=1)
   if(!isTRUE(is.numeric(alpha) && 0.5 <= alpha && alpha <= 1)) {
@@ -355,42 +367,67 @@ sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"),
     fit$finalModel <- finalModel
     class(fit) <- c("perrySparseLTS", class(fit))
   } else {
-    initial <- match.arg(initial)
-    if(h < d[2] && initial == "hyperplane") initial <- "sparse"
-    if(!is.null(seed)) set.seed(seed)  # set seed of random number generator
-    
-    ## the same initial subsets are used for all values of lambda, except when 
-    ## the initial subsets are based on lasso solutions with 3 observations 
-    ## (since in the latter case the initial subsets depend on lambda)
-    subsets <- switch(initial, random=randomSubsets(n, h, nsamp[1]), 
-                      hyperplane=hyperplaneSubsets(x, y, h, nsamp[1]))
-    
-    ## call internal function to obtain raw fits
-    if(length(lambda) == 1) {
-      fit <- fastSparseLTS(x=x, y=y, lambda=lambda, h=h, nsamp=nsamp, 
-                           initial=subsets, intercept=intercept, ncstep=ncstep, 
-                           tol=tol, eps=eps, use.Gram=use.Gram, ncores=ncores)
-      fit$best <- sort.int(fit$best)
+    if(h < n) {
+      initial <- match.arg(initial)
+      if(h < d[2] && initial == "hyperplane") initial <- "sparse"
+      if(!is.null(seed)) set.seed(seed)  # set seed of random number generator
+      
+      ## the same initial subsets are used for all values of lambda, except when 
+      ## the initial subsets are based on lasso solutions with 3 observations 
+      ## (since in the latter case the initial subsets depend on lambda)
+      subsets <- switch(initial, random=randomSubsets(n, h, nsamp[1]), 
+                        hyperplane=hyperplaneSubsets(x, y, h, nsamp[1]))
+      
+      ## call internal function to obtain raw fits
+      if(length(lambda) == 1) {
+        fit <- fastSparseLTS(x=x, y=y, lambda=lambda, h=h, nsamp=nsamp, 
+                             initial=subsets, normalize=normalize, 
+                             intercept=intercept, ncstep=ncstep, tol=tol, 
+                             eps=eps, use.Gram=use.Gram, ncores=ncores)
+        fit$best <- sort.int(fit$best)
+      } else {
+        names(lambda) <- seq_along(lambda)
+        fit <- lapply(lambda, fastSparseLTS, x=x, y=y, h=h, nsamp=nsamp, 
+                      initial=subsets, normalize=normalize, intercept=intercept, 
+                      ncstep=ncstep, tol=tol, eps=eps, use.Gram=use.Gram, 
+                      ncores=ncores, drop=FALSE)
+        names(fit) <- names(lambda)
+        fit <- list(best=sapply(fit, function(x) sort.int(x$best)), 
+                    coefficients=sapply(fit, "[[", "coefficients"), 
+                    residuals=sapply(fit, "[[", "residuals"), 
+                    objective=sapply(fit, "[[", "objective"), 
+                    center=sapply(fit, "[[", "center"), 
+                    scale=sapply(fit, "[[", "scale"))
+      }
+      ## compute consistency factor to correct scale estimate
+      qn <- qnorm((h+n)/ (2*n))                         # required quantile
+      cdelta <- 1 / sqrt(1 - (2*n)/(h/qn) * dnorm(qn))  # consistency factor
     } else {
-      names(lambda) <- seq_along(lambda)
-      fit <- lapply(lambda, fastSparseLTS, x=x, y=y, h=h, nsamp=nsamp, 
-                    initial=subsets, intercept=intercept, ncstep=ncstep, 
-                    tol=tol, eps=eps, use.Gram=use.Gram, ncores=ncores, 
-                    drop=FALSE)
-      names(fit) <- names(lambda)
-      fit <- list(best=sapply(fit, function(x) sort.int(x$best)), 
-                  coefficients=sapply(fit, "[[", "coefficients"), 
-                  residuals=sapply(fit, "[[", "residuals"), 
-                  objective=sapply(fit, "[[", "objective"), 
-                  center=sapply(fit, "[[", "center"), 
-                  scale=sapply(fit, "[[", "scale"))
+      ## no trimming, compute lasso for raw fits
+      if(length(lambda) == 1) {
+        fit <- fastLasso(x, y, lambda=lambda, normalize=normalize, 
+                         intercept=intercept, eps=eps, use.Gram=use.Gram, 
+                         raw=TRUE)
+      } else {
+        names(lambda) <- seq_along(lambda)
+        fit <- lapply(lambda, fastLasso, x=x, y=y, normalize=normalize, 
+                      intercept=intercept, eps=eps, use.Gram=use.Gram, 
+                      drop=FALSE, raw=TRUE)
+        names(fit) <- names(lambda)
+        fit <- list(best=sapply(fit, "[[", "best"), 
+                    coefficients=sapply(fit, "[[", "coefficients"), 
+                    residuals=sapply(fit, "[[", "residuals"), 
+                    objective=sapply(fit, "[[", "objective"), 
+                    center=sapply(fit, "[[", "center"), 
+                    scale=sapply(fit, "[[", "scale"))
+      }
+      ## consistency factor is not necessary
+      cdelta <- 1
     }
-    ## correct scale estimate
-    q <- qnorm(0.9875)  # quantile of the normal distribution
-    qn <- qnorm((h+n)/ (2*n))  # quantile for consistency factor
-    cdelta <- 1 / sqrt(1 - (2*n)/(h/qn) * dnorm(qn))  # consistency factor
-    s <- fit$scale * cdelta
+    
     ## find good observations
+    q <- qnorm(0.9875)       # quantile of the normal distribution
+    s <- fit$scale * cdelta  # corrected scale estimate
     if(length(lambda) == 1) ok <- abs((fit$residuals - fit$center)/s) <= q
     else ok <- abs(scale(fit$residuals, fit$center, s)) <= q
     ## convert to 0/1 weights identifying outliers
@@ -407,13 +444,13 @@ sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"),
     # compute lasso fits with good observations
     if(length(lambda) == 1) {
       fit <- fastLasso(x, y, lambda=lambda, subset=which(ok), 
-                       intercept=intercept, eps=eps, 
-                       use.Gram=use.Gram)
+                       normalize=normalize, intercept=intercept, 
+                       eps=eps, use.Gram=use.Gram)
     } else {
       fit <- lapply(seq_along(lambda), function(i) {
         fastLasso(x, y, lambda=lambda[i], subset=which(ok[, i]), 
-                  intercept=intercept, eps=eps, 
-                  use.Gram=use.Gram, drop=FALSE)
+                  normalize=normalize, intercept=intercept, 
+                  eps=eps, use.Gram=use.Gram, drop=FALSE)
       })
       names(fit) <- names(lambda)
       fit <- list(coefficients=sapply(fit, "[[", "coefficients"), 
@@ -456,7 +493,7 @@ sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"),
                 fitted.values=copyNames(from=y, to=fit$fitted.values), 
                 residuals=copyNames(from=y, to=fit$residuals), center=center, 
                 scale=s, cnp2=cdelta, wt=copyNames(from=y, to=wt), df=df, 
-                intercept=intercept, alpha=alpha, quan=h, 
+                normalize=normalize, intercept=intercept, alpha=alpha, quan=h, 
                 raw.coefficients=copyNames(from=x, to=raw.fit$coefficients), 
                 raw.fitted.values=copyNames(from=y, to=y-raw.fit$residuals),
                 raw.residuals=copyNames(from=y, to=raw.fit$residuals), 
@@ -478,20 +515,21 @@ sparseLTS.default <- function(x, y, lambda, mode = c("lambda", "fraction"),
 
 ## internal function to compute raw sparse LTS
 fastSparseLTS <- function(lambda, x, y, h, nsamp = c(500, 10), 
-                          initial = NULL, intercept = TRUE, 
+                          initial = NULL, normalize = TRUE, intercept = TRUE, 
                           ncstep = 2, tol = .Machine$double.eps^0.5, 
                           eps = .Machine$double.eps, use.Gram = TRUE, 
                           ncores = 1, drop = TRUE) {
   # check whether initial subsets based on lasso solutions need to be computed
   if(is.null(initial)) {
     initial <- sparseSubsets(x, y, lambda=lambda, h=h, nsamp=nsamp[1], 
-                             intercept=intercept, eps=eps, use.Gram=use.Gram)
+                             normalize=normalize, intercept=intercept, 
+                             eps=eps, use.Gram=use.Gram)
   }
   # call C++ function
   fit <- callBackend("R_fastSparseLTS", R_x=x, R_y=y, R_lambda=lambda, 
-                     R_initial=initial, R_intercept=intercept, R_ncstep=ncstep, 
-                     R_nkeep=nsamp[2], R_tol=tol, R_eps=eps, R_useGram=use.Gram, 
-                     R_ncores=ncores)
+                     R_initial=initial, R_normalize=normalize, 
+                     R_intercept=intercept, R_ncstep=ncstep, R_nkeep=nsamp[2], 
+                     R_tol=tol, R_eps=eps, R_useGram=use.Gram, R_ncores=ncores)
   if(drop) {
     # drop the dimension of selected components
     which <- c("best", "coefficients", "residuals")
